#include <QFile>
#include <QMessageBox>
#include <QListWidgetItem>
#include <QDebug>

#include "mainwindow.h"
#include "ui_mainwindow.h"

#include "efd400.h"

MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent), ui(new Ui::MainWindow), m_init(false)
{
	ui->setupUi(this);

	m_lift.load(":/lift.png");
	m_press.load(":/pressdown.png");

	m_statusInfo = new QLabel(this);
	QFont font	 = m_statusInfo->font();
	font.setPointSize(18);
	m_statusInfo->setFont(font);
	ui->statusBar->addWidget(m_statusInfo);

	ui->textEdit->setReadOnly(true);

	connect(this, &MainWindow::startTemplate, &m_tasks, &Tasks::startTemplate);
	connect(this, &MainWindow::startComparison, &m_tasks, &Tasks::startComparison);
	connect(this, &MainWindow::startCaptureImage, &m_tasks, &Tasks::startCaptureImage);
	connect(this, &MainWindow::stopCaptureImage, &m_tasks, &Tasks::stopCaptureImage);
	connect(&m_tasks, &Tasks::sendCaptureImage, this, &MainWindow::showImage);
	connect(&m_tasks, &Tasks::sendDeviceInfo, this, &MainWindow::showDeviceInfo);
	connect(&m_tasks, &Tasks::sendTemplateResults, this, &MainWindow::templateResults);
	connect(&m_tasks, &Tasks::sendComparisonResults, this, &MainWindow::comparisonResults);

	setShowButton(0);
}

MainWindow::~MainWindow() { delete ui; }

void MainWindow::on_pushButton_clicked()
{
	if(m_init)
		return;

	int ret;

	ret = m_tasks.openDevice();
	qDebug() << "open device = " << ret;
	if(ret == EVD_SUCC)
	{
		m_init = true;
		m_statusInfo->setText("打开设备成功");
		setShowButton(1);
	}
	else
		m_statusInfo->setText("打开设备失败");
}

void MainWindow::on_pushButton_2_clicked()
{
	if(!m_init)
		return;

	emit startCaptureImage();
	m_collection = true;
	m_statusInfo->setText("开始采集");
	setShowButton(2);
}

void MainWindow::on_pushButton_3_clicked()
{
	if(!m_init)
		return;

	m_tasks.closeDevice();
	m_init		 = false;
	m_collection = false;
	ui->showpic->clear();
	m_statusInfo->setText("关闭设备");
	setShowButton(0);
	ui->textEdit->setText("");
}

void MainWindow::on_pushButton_4_clicked()
{
	if(m_collection)
	{
		m_statusInfo->setText("采集生成模板");
		emit startTemplate();
	}
}

void MainWindow::on_pushButton_5_clicked()
{
	if(m_collection)
	{
		m_statusInfo->setText("采集指纹并与模板比对");
		emit startComparison();
	}
}

void MainWindow::on_pushButton_6_clicked()
{
	if(!m_init)
		return;

	emit stopCaptureImage();
	ui->showpic->clear();
	m_collection = false;
	m_statusInfo->setText("停止采集");
	setShowButton(1);
}

void MainWindow::showImage(QPixmap image)
{
	if(image.width() > 0)
		ui->showpic->setPixmap(image);
}

void MainWindow::showDeviceInfo(QString info)
{
	qDebug() << "info = " << info;
	ui->textEdit->setText(info);
}

void MainWindow::templateResults(bool res)
{
	m_statusInfo->setText(res ? "生成模板成功" : "生成模板失败");
}

void MainWindow::comparisonResults(bool res)
{
	m_statusInfo->setText(res ? "比对成功" : "比对失败");
}

void MainWindow::setShowButton(int s)
{
	if(s == 0)	//默认
	{
		ui->pushButton->setEnabled(true);
		ui->pushButton_3->setEnabled(false);

		ui->pushButton_2->setEnabled(false);
		ui->pushButton_6->setEnabled(false);

		ui->pushButton_4->setEnabled(false);
		ui->pushButton_5->setEnabled(false);
	}
	else if(s == 1)	 //打开设备&关闭采集
	{
		ui->pushButton->setEnabled(false);
		ui->pushButton_3->setEnabled(true);

		ui->pushButton_2->setEnabled(true);
		ui->pushButton_6->setEnabled(false);

		ui->pushButton_4->setEnabled(false);
		ui->pushButton_5->setEnabled(false);
	}
	else if(s == 2)	 //开始采集
	{
		ui->pushButton->setEnabled(false);
		ui->pushButton_3->setEnabled(true);

		ui->pushButton_2->setEnabled(false);
		ui->pushButton_6->setEnabled(true);

		ui->pushButton_4->setEnabled(true);
		ui->pushButton_5->setEnabled(true);
	}
}
