#include <QImage>
#include <QThread>
#include <QDebug>

#include "tasks.h"
#include "ID_Fpr.h"

#define IMAGE_WIDTH 256
#define IMAGE_HEIGHT 360
#define COMP_MAGIC 0.60f

Tasks::Tasks(QObject *parent) : QObject(parent), m_startComparison(false), m_startTemplate(false)
{
	m_count		  = 0;
	m_singleCount = 0;

	for(int i = 0; i < 3; i++)
	{
		//		m_image[i].image_data = new unsigned char[IMAGE_WIDTH * IMAGE_HEIGHT];
	}

	m_captureTimer.setInterval(30);
	m_captureTimer.setSingleShot(false);
	connect(&m_captureTimer, &QTimer::timeout, this, &Tasks::captureImageTask);
}

Tasks::~Tasks()
{
	closeDevice();

	//	for(int i = 0; i < 3; i++) delete[] m_image[i].image_data;
}

int Tasks::openDevice()
{
	int ret = LIVESCAN_Init();
	qDebug() << "init" << ret;

	if(ret == EVD_SUCC)
	{
		char sn[64] = {0};
		ret			= LIVESCAN_GetDeviceSerialNumber(0, (unsigned char *)sn);
		qDebug() << "get sn = " << ret;
		if(EVD_SUCC == ret)
		{
			QString infoText = QString("sn: %1").arg(sn);
			emit sendDeviceInfo(infoText);
		}

		char desc[1024] = {0};
		ret = LIVESCAN_GetDesc(desc);
		qDebug() << "get desc = " << ret;
		if(EVD_SUCC == ret)
		{}

		ret = FP_Begin();
		if(ret == FP_ERR_SUCCESS)
		{
			char ver[5] = {0};
			FP_GetVersion((unsigned char *)ver);
			qDebug() << "finger alg version = " << ver;
		}
	}

	return ret;
}

void Tasks::closeDevice()
{
	stopCaptureImage();
	LIVESCAN_Close();

	FP_End();
}

void Tasks::startCaptureImage()
{
	if(!m_captureTimer.isActive())
		m_captureTimer.start();
}

void Tasks::stopCaptureImage()
{
	if(m_captureTimer.isActive())
		m_captureTimer.stop();
}

void Tasks::startComparison()
{
	if(!m_startComparison)
	{
		m_startComparison = true;
	}
}

void Tasks::startTemplate()
{
	if(!m_startTemplate)
	{
		memset(m_feature, 0, sizeof(m_feature));
		m_startTemplate = true;
	}
}

void Tasks::captureImageTask()
{
	unsigned char buf[IMAGE_WIDTH * IMAGE_HEIGHT];
	memset(buf, 0, IMAGE_WIDTH * IMAGE_HEIGHT);
	int valid;
	static int press = 0;
	static int count = 0;

	int ret = LIVESCAN_GetFPRawData(0, buf);

	if(ret == EVD_SUCC)
	{
		QImage image((uchar *)buf, IMAGE_WIDTH, IMAGE_HEIGHT, QImage::Format_Grayscale8);
		emit sendCaptureImage(QPixmap::fromImage(image));

		if(m_startComparison)
		{
			//			if(count++ > 5 && valid == 1)
			//			{
			//				m_image[0].image_size = IMAGE_WIDTH * IMAGE_HEIGHT;
			//				memcpy(m_image[0].image_data, buf, m_image->image_size);
			//				if(ECGetFeatureByImage(m_image[0], m_feature, &m_feaSize) == EC_Success)
			//				{
			//					double score = ECMatchFingerVein(m_tmplate, m_tmplSize, m_feature, m_feaSize);
			//					emit sendComparisonResults(score >= COMP_MAGIC ? true : false);
			//				}
			//				count			  = 0;
			//				m_startComparison = false;
			//			}

			if(count <= 100)
			{
				count++;
				unsigned char score[1] = {0};
				ret					   = FP_GetQualityScore(buf, score);
				printf("score = %d\n", score[0]);
				if(ret != FP_ERR_SUCCESS || score[0] <= 50)
					return;

				unsigned char fea[512] = {0};

				ret = FP_FeatureExtract(0, 0, buf, fea);
				if(ret == FP_ERR_SUCCESS)
				{
					float similarity = 0.0f;

					ret = FP_FeatureMatch(fea, m_feature, &similarity);
					printf("FPM = %d\n", ret);
					if(ret == FP_ERR_SUCCESS)
					{
						count			  = 0;
						m_startComparison = false;
						printf("similarity = %f\n", similarity);
						emit sendComparisonResults(similarity > COMP_MAGIC ? true : false);
					}
				}

				QThread::usleep(1000);
			}
			else
			{
				count			  = 0;
				m_startComparison = false;
				emit sendComparisonResults(false);
			}
		}
		else if(m_startTemplate)
		{
			if(count <= 100)
			{
				count++;
				unsigned char score[1] = {0};
				ret					   = FP_GetQualityScore(buf, score);
				printf("score = %d\n", score[0]);
				if(ret != FP_ERR_SUCCESS || score[0] <= 50)
					return;

				ret = FP_FeatureExtract(0, 0, buf, m_feature);
				if(ret == FP_ERR_SUCCESS)
				{
					emit sendTemplateResults(true);

					count			= 0;
					m_startTemplate = false;

					return;
				}

				QThread::usleep(1000);
			}
			else
			{
				emit sendTemplateResults(false);
				m_startTemplate = false;
			}
		}
	}
	else
		emit sendCaptureImage(QPixmap());
}
